@@ -281,14 +281,15 @@ void mutt_query_exit (void)
 void mutt_curses_error (const char *fmt, ...)
 {
   va_list ap;
+  char scratch[LONG_STRING];
 
   va_start (ap, fmt);
-  vsnprintf (Errorbuf, sizeof (Errorbuf), fmt, ap);
+  vsnprintf (scratch, sizeof (scratch), fmt, ap);
   va_end (ap);
   
-  dprint (1, (debugfile, "%s\n", Errorbuf));
+  dprint (1, (debugfile, "%s\n", scratch));
   mutt_format_string (Errorbuf, sizeof (Errorbuf),
-		      0, COLS-2, 0, 0, Errorbuf, sizeof (Errorbuf), 0);
+		      0, COLS-2, 0, 0, scratch, sizeof (scratch), 0);
 
   if (!option (OPTKEEPQUIET))
   {
@@ -306,13 +307,14 @@ void mutt_curses_error (const char *fmt, ...)
 void mutt_curses_message (const char *fmt, ...)
 {
   va_list ap;
+  char scratch[LONG_STRING];
 
   va_start (ap, fmt);
-  vsnprintf (Errorbuf, sizeof (Errorbuf), fmt, ap);
+  vsnprintf (scratch, sizeof (scratch), fmt, ap);
   va_end (ap);
 
   mutt_format_string (Errorbuf, sizeof (Errorbuf),
-		      0, COLS-2, 0, 0, Errorbuf, sizeof (Errorbuf), 0);
+		      0, COLS-2, 0, 0, scratch, sizeof (scratch), 0);
 
   if (!option (OPTKEEPQUIET))
   {
@@ -326,37 +328,72 @@ void mutt_curses_message (const char *fmt, ...)
   unset_option (OPTMSGERR);
 }
 
-#ifdef USE_SOCKET
-void mutt_progress_bar (progress_t* progress, long pos)
+void mutt_progress_init (progress_t* progress, const char *msg,
+			 unsigned short flags, unsigned short inc,
+			 long size)
+{
+  if (!progress)
+    return;
+  memset (progress, 0, sizeof (progress_t));
+  progress->inc = inc;
+  progress->flags = flags;
+  progress->msg = msg;
+  progress->size = size;
+  mutt_progress_update (progress, 0);
+}
+
+void mutt_progress_update (progress_t* progress, long pos)
 {
   char posstr[SHORT_STRING];
+  short update = 0;
 
   if (!pos)
   {
-    if (!NetInc)
+    if (!progress->inc)
       mutt_message (progress->msg);
-    else {
+    else
+    {
       if (progress->size)
-	mutt_pretty_size (progress->sizestr, sizeof (progress->sizestr), progress->size);
+      {
+	if (progress->flags & M_PROGRESS_SIZE)
+	  mutt_pretty_size (progress->sizestr, sizeof (progress->sizestr), progress->size);
+	else
+	  snprintf (progress->sizestr, sizeof (progress->sizestr), "%ld", progress->size);
+      }
       progress->pos = 0;
     }
   }
 
-  if (!NetInc)
+  if (!progress->inc)
     return;
 
-  if (pos >= progress->pos + (NetInc << 10))
+  if (progress->flags & M_PROGRESS_SIZE)
+  {
+    if (pos >= progress->pos + (progress->inc << 10))
+    {
+      pos = pos / (progress->inc << 10) * (progress->inc << 10);
+      mutt_pretty_size (posstr, sizeof (posstr), pos);
+      update = 1;
+    }
+  }
+  else if (pos >= progress->pos + progress->inc)
+  {
+    snprintf (posstr, sizeof (posstr), "%ld", pos);
+    update = 1;
+  }
+
+  if (update)
   {
     progress->pos = pos;
-    pos = pos / (NetInc << 10) * (NetInc << 10);
-    mutt_pretty_size (posstr, sizeof (posstr), pos);
     if (progress->size)
       mutt_message ("%s %s/%s", progress->msg, posstr, progress->sizestr);
     else
       mutt_message ("%s %s", progress->msg, posstr);
   }
+
+  if (pos >= progress->size)
+    mutt_clear_error ();
 }
-#endif
 
 void mutt_show_error (void)
 {
@@ -620,6 +657,9 @@ void mutt_format_string (char *dest, size_t destlen,
   {
     if (k == (size_t)(-1) || k == (size_t)(-2))
     {
+      if (k == (size_t)(-1) && errno == EILSEQ)
+	memset (&mbstate1, 0, sizeof (mbstate1));
+
       k = (k == (size_t)(-1)) ? 1 : n;
       wc = replacement_char ();
     }
@@ -729,6 +769,8 @@ void mutt_paddstr (int n, const char *s)
   {
     if (k == (size_t)(-1) || k == (size_t)(-2))
     {
+      if (k == (size_t) (-1))
+	memset (&mbstate, 0, sizeof (mbstate));
       k = (k == (size_t)(-1)) ? 1 : len;
       wc = replacement_char ();
     }
