@@ -722,38 +722,38 @@ static void cmd_parse_myrights (IMAP_DATA* idata, const char* s)
   s = imap_next_word ((char*)s);
 
   /* zero out current rights set */
-  memset (idata->rights, 0, sizeof (idata->rights));
+  memset (idata->ctx->rights, 0, sizeof (idata->ctx->rights));
 
   while (*s && !isspace((unsigned char) *s))
   {
     switch (*s) 
     {
       case 'l':
-	mutt_bit_set (idata->rights, IMAP_ACL_LOOKUP);
+	mutt_bit_set (idata->ctx->rights, M_ACL_LOOKUP);
 	break;
       case 'r':
-	mutt_bit_set (idata->rights, IMAP_ACL_READ);
+	mutt_bit_set (idata->ctx->rights, M_ACL_READ);
 	break;
       case 's':
-	mutt_bit_set (idata->rights, IMAP_ACL_SEEN);
+	mutt_bit_set (idata->ctx->rights, M_ACL_SEEN);
 	break;
       case 'w':
-	mutt_bit_set (idata->rights, IMAP_ACL_WRITE);
+	mutt_bit_set (idata->ctx->rights, M_ACL_WRITE);
 	break;
       case 'i':
-	mutt_bit_set (idata->rights, IMAP_ACL_INSERT);
+	mutt_bit_set (idata->ctx->rights, M_ACL_INSERT);
 	break;
       case 'p':
-	mutt_bit_set (idata->rights, IMAP_ACL_POST);
+	mutt_bit_set (idata->ctx->rights, M_ACL_POST);
 	break;
       case 'c':
-	mutt_bit_set (idata->rights, IMAP_ACL_CREATE);
+	mutt_bit_set (idata->ctx->rights, M_ACL_CREATE);
 	break;
       case 'd':
-	mutt_bit_set (idata->rights, IMAP_ACL_DELETE);
+	mutt_bit_set (idata->ctx->rights, M_ACL_DELETE);
 	break;
       case 'a':
-	mutt_bit_set (idata->rights, IMAP_ACL_ADMIN);
+	mutt_bit_set (idata->ctx->rights, M_ACL_ADMIN);
 	break;
     }
     s++;
