@@ -223,7 +223,7 @@ int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes, progress_t* pbar)
     fputc (c, fp);
     
     if (pbar && !(pos % 1024))
-      mutt_progress_bar (pbar, pos);
+      mutt_progress_update (pbar, pos);
 #ifdef DEBUG
     if (debuglevel >= IMAP_LOG_LTRL)
       fputc (c, debugfile);
@@ -573,7 +573,7 @@ int imap_open_mailbox (CONTEXT* ctx)
 
   /* clear mailbox status */
   idata->status = 0;
-  memset (idata->rights, 0, (RIGHTSMAX+7)/8);
+  memset (idata->ctx->rights, 0, sizeof (idata->ctx->rights));
   idata->newMailCount = 0;
 
   mutt_message (_("Selecting %s..."), idata->mailbox);
@@ -588,14 +588,14 @@ int imap_open_mailbox (CONTEXT* ctx)
   /* assume we have all rights if ACL is unavailable */
   else
   {
-    mutt_bit_set (idata->rights, IMAP_ACL_LOOKUP);
-    mutt_bit_set (idata->rights, IMAP_ACL_READ);
-    mutt_bit_set (idata->rights, IMAP_ACL_SEEN);
-    mutt_bit_set (idata->rights, IMAP_ACL_WRITE);
-    mutt_bit_set (idata->rights, IMAP_ACL_INSERT);
-    mutt_bit_set (idata->rights, IMAP_ACL_POST);
-    mutt_bit_set (idata->rights, IMAP_ACL_CREATE);
-    mutt_bit_set (idata->rights, IMAP_ACL_DELETE);
+    mutt_bit_set (idata->ctx->rights, M_ACL_LOOKUP);
+    mutt_bit_set (idata->ctx->rights, M_ACL_READ);
+    mutt_bit_set (idata->ctx->rights, M_ACL_SEEN);
+    mutt_bit_set (idata->ctx->rights, M_ACL_WRITE);
+    mutt_bit_set (idata->ctx->rights, M_ACL_INSERT);
+    mutt_bit_set (idata->ctx->rights, M_ACL_POST);
+    mutt_bit_set (idata->ctx->rights, M_ACL_CREATE);
+    mutt_bit_set (idata->ctx->rights, M_ACL_DELETE);
   }
   /* pipeline the postponed count if possible */
   pmx.mbox = NULL;
@@ -724,10 +724,10 @@ int imap_open_mailbox (CONTEXT* ctx)
   }
 #endif
 
-  if (!(mutt_bit_isset(idata->rights, IMAP_ACL_DELETE) ||
-        mutt_bit_isset(idata->rights, IMAP_ACL_SEEN) ||
-        mutt_bit_isset(idata->rights, IMAP_ACL_WRITE) ||
-        mutt_bit_isset(idata->rights, IMAP_ACL_INSERT)))
+  if (!(mutt_bit_isset(idata->ctx->rights, M_ACL_DELETE) ||
+        mutt_bit_isset(idata->ctx->rights, M_ACL_SEEN) ||
+        mutt_bit_isset(idata->ctx->rights, M_ACL_WRITE) ||
+        mutt_bit_isset(idata->ctx->rights, M_ACL_INSERT)))
      ctx->readonly = 1;
 
   ctx->hdrmax = count;
@@ -815,7 +815,7 @@ void imap_logout (IMAP_DATA* idata)
 static void imap_set_flag (IMAP_DATA* idata, int aclbit, int flag,
   const char *str, char *flags, size_t flsize)
 {
-  if (mutt_bit_isset (idata->rights, aclbit))
+  if (mutt_bit_isset (idata->ctx->rights, aclbit))
     if (flag && imap_has_flag (idata->flags, str))
       safe_strcat (flags, flsize, str);
 }
@@ -985,19 +985,19 @@ int imap_sync_message (IMAP_DATA *idata, HEADER *hdr, BUFFER *cmd,
 
   flags[0] = '\0';
       
-  imap_set_flag (idata, IMAP_ACL_SEEN, hdr->read, "\\Seen ",
+  imap_set_flag (idata, M_ACL_SEEN, hdr->read, "\\Seen ",
 		 flags, sizeof (flags));
-  imap_set_flag (idata, IMAP_ACL_WRITE, hdr->old,
+  imap_set_flag (idata, M_ACL_WRITE, hdr->old,
                  "Old ", flags, sizeof (flags));
-  imap_set_flag (idata, IMAP_ACL_WRITE, hdr->flagged,
+  imap_set_flag (idata, M_ACL_WRITE, hdr->flagged,
 		 "\\Flagged ", flags, sizeof (flags));
-  imap_set_flag (idata, IMAP_ACL_WRITE, hdr->replied,
+  imap_set_flag (idata, M_ACL_WRITE, hdr->replied,
 		 "\\Answered ", flags, sizeof (flags));
-  imap_set_flag (idata, IMAP_ACL_DELETE, hdr->deleted,
+  imap_set_flag (idata, M_ACL_DELETE, hdr->deleted,
 		 "\\Deleted ", flags, sizeof (flags));
 
   /* now make sure we don't lose custom tags */
-  if (mutt_bit_isset (idata->rights, IMAP_ACL_WRITE))
+  if (mutt_bit_isset (idata->ctx->rights, M_ACL_WRITE))
     imap_add_keywords (flags, hdr, idata->flags, sizeof (flags));
 
   mutt_remove_trailing_ws (flags);
@@ -1006,11 +1006,11 @@ int imap_sync_message (IMAP_DATA *idata, HEADER *hdr, BUFFER *cmd,
    * explicitly revoke all system flags (if we have permission) */
   if (!*flags)
   {
-    imap_set_flag (idata, IMAP_ACL_SEEN, 1, "\\Seen ", flags, sizeof (flags));
-    imap_set_flag (idata, IMAP_ACL_WRITE, 1, "Old ", flags, sizeof (flags));
-    imap_set_flag (idata, IMAP_ACL_WRITE, 1, "\\Flagged ", flags, sizeof (flags));
-    imap_set_flag (idata, IMAP_ACL_WRITE, 1, "\\Answered ", flags, sizeof (flags));
-    imap_set_flag (idata, IMAP_ACL_DELETE, 1, "\\Deleted ", flags, sizeof (flags));
+    imap_set_flag (idata, M_ACL_SEEN, 1, "\\Seen ", flags, sizeof (flags));
+    imap_set_flag (idata, M_ACL_WRITE, 1, "Old ", flags, sizeof (flags));
+    imap_set_flag (idata, M_ACL_WRITE, 1, "\\Flagged ", flags, sizeof (flags));
+    imap_set_flag (idata, M_ACL_WRITE, 1, "\\Answered ", flags, sizeof (flags));
+    imap_set_flag (idata, M_ACL_DELETE, 1, "\\Deleted ", flags, sizeof (flags));
 
     mutt_remove_trailing_ws (flags);
 
@@ -1046,10 +1046,10 @@ static int sync_helper (IMAP_DATA* idata, BUFFER* buf, int right, int flag,
 {
   int rc = 0;
 
-  if (!mutt_bit_isset (idata->rights, right))
+  if (!mutt_bit_isset (idata->ctx->rights, right))
     return 0;
   
-  if (right == IMAP_ACL_WRITE && !imap_has_flag (idata->flags, name))
+  if (right == M_ACL_WRITE && !imap_has_flag (idata->flags, name))
     return 0;
 
   buf->dptr = buf->data;
@@ -1111,7 +1111,7 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
   memset (&cmd, 0, sizeof (cmd));
 
   /* if we are expunging anyway, we can do deleted messages very quickly... */
-  if (expunge && mutt_bit_isset (idata->rights, IMAP_ACL_DELETE))
+  if (expunge && mutt_bit_isset (idata->ctx->rights, M_ACL_DELETE))
   {
     mutt_buffer_addstr (&cmd, "UID STORE ");
     deleted = imap_make_msg_set (idata, &cmd, M_DELETED, 1, 0);
@@ -1194,11 +1194,11 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
            mutt_get_sort_func (SORT_ORDER));
   }
 
-  rc += sync_helper (idata, &cmd, IMAP_ACL_DELETE, M_DELETED, "\\Deleted");
-  rc += sync_helper (idata, &cmd, IMAP_ACL_WRITE, M_FLAG, "\\Flagged");
-  rc += sync_helper (idata, &cmd, IMAP_ACL_WRITE, M_OLD, "Old");
-  rc += sync_helper (idata, &cmd, IMAP_ACL_SEEN, M_READ, "\\Seen");
-  rc += sync_helper (idata, &cmd, IMAP_ACL_WRITE, M_REPLIED, "\\Answered");
+  rc += sync_helper (idata, &cmd, M_ACL_DELETE, M_DELETED, "\\Deleted");
+  rc += sync_helper (idata, &cmd, M_ACL_WRITE, M_FLAG, "\\Flagged");
+  rc += sync_helper (idata, &cmd, M_ACL_WRITE, M_OLD, "Old");
+  rc += sync_helper (idata, &cmd, M_ACL_SEEN, M_READ, "\\Seen");
+  rc += sync_helper (idata, &cmd, M_ACL_WRITE, M_REPLIED, "\\Answered");
 
   if (oldsort != Sort)
   {
@@ -1231,7 +1231,7 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
 
   /* We must send an EXPUNGE command if we're not closing. */
   if (expunge && !(ctx->closing) &&
-      mutt_bit_isset(idata->rights, IMAP_ACL_DELETE))
+      mutt_bit_isset(idata->ctx->rights, M_ACL_DELETE))
   {
     mutt_message _("Expunging messages from server...");
     /* Set expunge bit so we don't get spurious reopened messages */
@@ -1300,7 +1300,9 @@ void imap_close_mailbox (CONTEXT* ctx)
 
   /* free IMAP part of headers */
   for (i = 0; i < ctx->msgcount; i++)
-    imap_free_header_data (&(ctx->hdrs[i]->data));
+    /* mailbox may not have fully loaded */
+    if (ctx->hdrs[i] && ctx->hdrs[i]->data)
+      imap_free_header_data (&(ctx->hdrs[i]->data));
 
   for (i = 0; i < IMAP_CACHE_LEN; i++)
   {
@@ -1732,7 +1734,7 @@ int imap_subscribe (char *path, int subscribe)
   BUFFER err, token;
   IMAP_MBOX mx;
 
-  if (!mx_is_imap (path) || imap_parse_path (path, &mx))
+  if (!mx_is_imap (path) || imap_parse_path (path, &mx) || !mx.mbox)
   {
     mutt_error (_("Bad mailbox name"));
     return -1;
