@@ -26,6 +26,8 @@
  * some of our "standard" functions in external programs, too.
  */
 
+#define _LIB_C 1
+
 #if HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -48,6 +50,7 @@
 
 #include "lib.h"
 
+
 static struct sysexits
 {
   int v;
@@ -361,7 +364,7 @@ int mutt_copy_stream (FILE *fin, FILE *fout)
   return 0;
 }
 
-static int 
+int 
 compare_stat (struct stat *osb, struct stat *nsb)
 {
   if (osb->st_dev != nsb->st_dev || osb->st_ino != nsb->st_ino ||
@@ -412,6 +415,8 @@ int safe_symlink(const char *oldpath, const char *newpath)
   return 0;
 }
 
+
+
 /* 
  * This function is supposed to do nfs-safe renaming of files.
  * 
@@ -439,10 +444,22 @@ int safe_rename (const char *src, const char *target)
      * to try it here.
      *
      */
+    
+    dprint (1, (debugfile, "safe_rename: link (%s, %s) failed: %s (%d)\n", src, target, strerror (errno), errno));
 
     if (errno == EXDEV)
-      return rename (src, target);
+    {
+      dprint (1, (debugfile, "safe_rename: errno was EXDEV; trying rename...\n"));
+      if (rename (src, target) == -1) 
+      {
+	dprint (1, (debugfile, "safe_rename: rename (%s, %s) failed: %s (%d)\n", src, target, strerror (errno), errno));
+	return -1;
+      }
+      dprint (1, (debugfile, "safe_rename: rename succeeded.\n"));
     
+      return 0;
+    }
+
     return -1;
   }
 
@@ -450,13 +467,17 @@ int safe_rename (const char *src, const char *target)
    * Stat both links and check if they are equal.
    */
   
-  if (stat (src, &ssb) == -1)
+  if (lstat (src, &ssb) == -1)
   {
+    dprint (1, (debugfile, "safe_rename: can't stat %s: %s (%d)\n",
+		src, strerror (errno), errno));
     return -1;
   }
   
-  if (stat (target, &tsb) == -1)
+  if (lstat (target, &tsb) == -1)
   {
+    dprint (1, (debugfile, "safe_rename: can't stat %s: %s (%d)\n",
+		src, strerror (errno), errno));
     return -1;
   }
 
@@ -467,6 +488,7 @@ int safe_rename (const char *src, const char *target)
 
   if (compare_stat (&ssb, &tsb) == -1)
   {
+    dprint (1, (debugfile, "safe_rename: stat blocks for %s and %s diverge; pretending EEXIST.\n", src, target));
     errno = EEXIST;
     return -1;
   }
@@ -476,19 +498,96 @@ int safe_rename (const char *src, const char *target)
    * value here? XXX
    */
 
-  unlink (src);
+  if (unlink (src) == -1) 
+  {
+    dprint (1, (debugfile, "safe_rename: unlink (%s) failed: %s (%d)\n",
+		src, strerror (errno), errno));
+  }
+  
 
   return 0;
 }
 
+
+/* Create a temporary directory next to a file name */
+
+int mutt_mkwrapdir (const char *path, char *newfile, size_t nflen, 
+		    char *newdir, size_t ndlen)
+{
+  const char *basename;
+  char parent[_POSIX_PATH_MAX];
+  char *p;
+  int rv;
+
+  strfcpy (parent, NONULL (path), sizeof (parent));
+  
+  if ((p = strrchr (parent, '/')))
+  {
+    *p = '\0';
+    basename = p + 1;
+  }
+  else
+  {
+    strfcpy (parent, ".", sizeof (parent));
+    basename = path;
+  }
+
+  do 
+  {
+    snprintf (newdir, ndlen, "%s/%s", parent, ".muttXXXXXX");
+    mktemp (newdir);
+  } 
+  while ((rv = mkdir (newdir, 0700)) == -1 && errno == EEXIST);
+  
+  if (rv == -1)
+    return -1;
+  
+  snprintf (newfile, nflen, "%s/%s", newdir, NONULL(basename));
+  return 0;  
+}
+
+int mutt_put_file_in_place (const char *path, const char *safe_file, const char *safe_dir)
+{
+  int rv;
+  
+  rv = safe_rename (safe_file, path);
+  unlink (safe_file);
+  rmdir (safe_dir);
+  return rv;
+}
+
 int safe_open (const char *path, int flags)
 {
   struct stat osb, nsb;
   int fd;
 
-  if ((fd = open (path, flags, 0600)) < 0)
-    return fd;
+  if (flags & O_EXCL) 
+  {
+    char safe_file[_POSIX_PATH_MAX];
+    char safe_dir[_POSIX_PATH_MAX];
 
+    if (mutt_mkwrapdir (path, safe_file, sizeof (safe_file),
+			safe_dir, sizeof (safe_dir)) == -1)
+      return -1;
+    
+    if ((fd = open (safe_file, flags, 0600)) < 0)
+    {
+      rmdir (safe_dir);
+      return fd;
+    }
+    
+    if (mutt_put_file_in_place (path, safe_file, safe_dir) == -1)
+    {
+      close (fd);
+      return -1;
+    }
+  }
+  else
+  {
+    if ((fd = open (path, flags, 0600)) < 0)
+      return fd;
+  }
+    
   /* make sure the file is not symlink */
   if (lstat (path, &osb) < 0 || fstat (fd, &nsb) < 0 ||
       compare_stat(&osb, &nsb) == -1)
