@@ -71,6 +71,8 @@
 #define INITVAL(x) 
 #endif
 
+#define WHERE_DEFINED 1
+
 #include "mutt_regex.h"
 
 /* flags for mutt_copy_header() */
@@ -831,6 +833,22 @@ typedef struct pattern_t
   } p;
 } pattern_t;
 
+/* ACL Rights */
+enum
+{
+  M_ACL_LOOKUP = 0,
+  M_ACL_READ,
+  M_ACL_SEEN,
+  M_ACL_WRITE,
+  M_ACL_INSERT,
+  M_ACL_POST,
+  M_ACL_CREATE,
+  M_ACL_DELETE,
+  M_ACL_ADMIN,
+
+  RIGHTSMAX
+};
+
 typedef struct
 {
   char *path;
@@ -863,6 +881,8 @@ typedef struct
 
   short magic;			/* mailbox type */
 
+  unsigned char rights[(RIGHTSMAX + 7)/8];	/* ACL bits */
+
   unsigned int locked : 1;	/* is the mailbox locked? */
   unsigned int changed : 1;	/* mailbox has been modified */
   unsigned int readonly : 1;    /* don't allow changes to the mailbox */
