@@ -15,8 +15,8 @@
  * 
  *     You should have received a copy of the GNU General Public
  *     License along with this program; if not, write to the Free
- *     Software Foundation, Inc., 59 Temple Place - Suite 330,
- *     Boston, MA  02111, USA.
+ *     Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *     Boston, MA  02110-1301, USA.
  */ 
 
 /*
@@ -126,7 +126,7 @@ static void mix_free_remailer (REMAILER **r)
   FREE (&(*r)->addr);
   FREE (&(*r)->ver);
   
-  FREE (r);
+  FREE (r);		/* __FREE_CHECKED__ */
 }
 
 /* parse the type2.list as given by mixmaster -T */
@@ -216,7 +216,7 @@ static void mix_free_type2_list (REMAILER ***ttlp)
   for (i = 0; type2_list[i]; i++)
     mix_free_remailer (&type2_list[i]);
   
-  FREE (type2_list);
+  FREE (type2_list);		/* __FREE_CHECKED__ */
 }
 
 
